function [lat, lon, alt, local_x, local_y, local_z] = getPosXPLANE(ip, port_Xplane, port_Matlab)
% Asensio Lorenzo 17/06/16
freq=16;

dataref{1,1}='sim/flightmodel/position/latitude';  dataref{1,2} = 140;
dataref{2,1}='sim/flightmodel/position/longitude'; dataref{2,2} = 141;
dataref{3,1}='sim/flightmodel/position/elevation'; dataref{3,2} = 142;
dataref{4,1}='sim/flightmodel/position/local_x';   dataref{4,2} = 143;
dataref{5,1}='sim/flightmodel/position/local_y';   dataref{5,2} = 144;
dataref{6,1}='sim/flightmodel/position/local_z';   dataref{6,2} = 145;

% Dataref Request (manual indexing)
dref_requestXPLANE(ip,port_Xplane,port_Matlab,freq,dataref);

% Dataref Reception (need to know the indexes, as several function may ask
% different datarefs)
[data, success] = dref_readXPLANE(port_Matlab, cell2mat(dataref(:,2)) );

if success
    lat = data(1);
    lon = data(2);
    alt = data(3);
    local_x = data(4);
    local_y = data(5);
    local_z = data(6);
else
    lat = NaN;
    lon = NaN;
    alt = NaN;
    local_x = NaN;
    local_y = NaN;
    local_z = NaN;
end
end


function sock = dref_requestXPLANE(ip,port_Xplane,port_Matlab,freq,dataref)
% Asensio Lorenzo 15/06/16

n_dref= size(dataref,1);
sock = java.net.DatagramSocket(port_Matlab);

% Message header (the same for every request)
intro_bytes = uint8(['RREF' 0]);
freq_bytes = typecast(uint32(freq),'uint8');
    
for i=1:n_dref
    dataref{i}(400) = 0; % Fixed length
    % Indexing
    if size(dataref,2) == 2
        index_bytes = typecast(uint32(dataref{i,2}),'uint8');
    else   
        index_bytes = typecast(uint32(i),'uint8');
    end
    % Message constructor
    send_msg = [intro_bytes freq_bytes index_bytes uint8(dataref{i,1})];
    send_pkt = java.net.DatagramPacket(send_msg, length(send_msg),...
               java.net.InetAddress.getByName(ip),port_Xplane);
    % Send message
    sock.send(send_pkt);
end
sock.close();
end


function [data, success] = dref_readXPLANE(port_Matlab, index) 
max_rep = 10;
success = false;

rep = 1;
while (rep <= max_rep && ~success)
    % Receive UDP data
    sock = java.net.DatagramSocket(port_Matlab);
    sock.setSoTimeout(2000);

    recv_pkt = java.net.DatagramPacket(uint8(zeros(1,256)), 256);
    sock.receive(recv_pkt);
    sock.close();
    
    recv_data = recv_pkt.getData();
    n = recv_pkt.getLength()-5;
    recv_data = recv_data(6:end);
    n = floor(n/8);
    
    % Define variables (without overwriting the available data)
    if ~exist('data','var')
         data = zeros(length(index),1);
    end      
    pindex = zeros(n,1);
    pdata  = zeros(n,1);
    
    % Import reived values
    for i=1:n
        pindex(i)= typecast(recv_data((i-1)*8+1:(i-1)*8+4),'uint32');
        pdata(i) = typecast(recv_data((i-1)*8+5:(i-1)*8+8),'single');
    end
    
    % Fill required data with received values
    len    = length(index);
    filled = zeros(1,len);
    for i = 1:len
        matching_case = pindex == index(i);
        if any(matching_case)
            data(i) = pdata(matching_case);
             filled(i) = true;
        end
    end
    
    % Detect if all required data has been received
    if all(filled)
        success = true;
    end
    
    rep = rep+1;
end
end