function setCoordXPLANE(ip, port_Xplane, port_Matlab, req_lat, req_lon, req_alt, wait_mode)
% Asensio Lorenzo 15/06/16

% req_lat / req_lon (degrees)
% req_alt           (ft) [optional]
% wait_mode = 'wait' waits until map is fully load [optional] 
%
% Asensio Lorenzo 17/06/16

long_pause = 1;
ft2m = 0.3048;
safe_local_y = 5000; % m
isBigJump = false;

%% Initialize
% Keep altitude if not defined
if (~exist('req_alt', 'var') || isempty(req_alt))
    [~, ~, xp_alt] = getXZ(ip, port_Xplane, port_Matlab); % 3rd output is Y
    req_alt = xp_alt/ft2m;
end

% Place the plane in the center of the sector (the plane is safer at a high
% altitude)
setPosXPLANE(ip,port_Xplane,0 ,safe_local_y,0);
pause(long_pause);
[xp_lat, xp_lon] = getLatLon(ip, port_Xplane, port_Matlab);
    
%% Sector Positioning (Rough)
% Set Rough Longitude
while (abs(req_lon - xp_lon) > 0.55)
    isBigJump = true;
    if (req_lon > xp_lon)
        moveFromSector(ip, port_Xplane, 'east');
    else
        moveFromSector(ip, port_Xplane, 'west');
    end
    setPosXPLANE(ip,port_Xplane,0 ,safe_local_y,0);
    pause(long_pause);
    [~, xp_lon] = getLatLon(ip, port_Xplane, port_Matlab);
end

% Set Rough Latitude
while (abs(req_lat - xp_lat) > 0.6)
    isBigJump = true;
    if (req_lat > xp_lat)
        moveFromSector(ip, port_Xplane, 'north');
    else
        moveFromSector(ip, port_Xplane, 'south');
    end
    setPosXPLANE(ip,port_Xplane,0 ,safe_local_y,0);
    pause(long_pause);
    [xp_lat, xp_lon] = getLatLon(ip, port_Xplane, port_Matlab);
end

if isBigJump
    fprintf('\tReloading scenary...');
    pause(10);
    fprintf('\tDone\n');
else
    setPosXPLANE(ip,port_Xplane,0 ,safe_local_y,0);
end

%% Coordinate Positioning (Fine)
% D: Desired / R: Random / O: Obtained / C: Calculated
tolerance = 0.001;
x_jump_R = 10000 * sign(req_lon - xp_lon );
z_jump_R = 10000 * sign(xp_lat  - req_lat);

while (abs(req_lat - xp_lat) > tolerance || abs(req_lon - xp_lon) > tolerance)
    % Set Fine Longitude
            start_lon  = xp_lon;
        [~, xp_lon] = moveWithinSector(ip, port_Xplane, port_Matlab, 'X', x_jump_R); fprintf('[setCoord] x_jump_R = %g\n',x_jump_R);
            diff_lon_R = xp_lon  - start_lon;
            diff_lon_D = req_lon - xp_lon;
            x_jump_C   = x_jump_R / diff_lon_R * diff_lon_D; % (Regla de 3)
            start_lon  = xp_lon;
        [xp_lat, xp_lon] = moveWithinSector(ip, port_Xplane, port_Matlab, 'X', x_jump_C); fprintf('[setCoord] x_jump_C = %g\n',x_jump_C);
            diff_lon_O = xp_lon - start_lon; % get
        
    % Set Fine Latitude
            start_lat = xp_lat;
        [xp_lat, ~] = moveWithinSector(ip, port_Xplane, port_Matlab, 'Z', z_jump_R); fprintf('[setCoord] z_jump_R = %g\n',z_jump_R);
            diff_lat_R = xp_lat  - start_lat;
            diff_lat_D = req_lat - xp_lat;
            z_jump_C   = z_jump_R / diff_lat_R * diff_lat_D; % (Regla de 3)
            start_lat  = xp_lat;
        [xp_lat, xp_lon] = moveWithinSector(ip, port_Xplane, port_Matlab, 'Z', z_jump_C); fprintf('[setCoord] z_jump_C = %g\n',z_jump_C);
            diff_lat_O = xp_lat - start_lat;
        
    % Recalculate first step jumps
        diff_lon_D = req_lon - xp_lon;
        x_jump_R = x_jump_C / diff_lon_O * diff_lon_D;
    	diff_lat_D = req_lat - xp_lat;
        z_jump_R = z_jump_C / diff_lat_O * diff_lat_D;
end

%% Restablish Heigh
[X, Z] = getXZ(ip, port_Xplane, port_Matlab);
setPosXPLANE(ip,port_Xplane,X,req_alt*ft2m,Z);

%% Wait until scenary is fully load (optional)
if exist('wait_mode', 'var') && strcmpi(wait_mode,'wait')
    while ~isScenaryLoadXPLANE(ip, port_Xplane, port_Matlab)
        pause(1);
    end
end
end

% Coordinates System
%  +------>   X
%  |
%  |
% \|/
%     Z

function moveFromSector(ip, port_Xplane, dir)
long_pause = 1;
jump = 75000;
safe_local_y = 5000; % m

switch dir
case {'up','north'}
    setPosXPLANE(ip,port_Xplane,0,safe_local_y,-jump);
case {'down','south'}
    setPosXPLANE(ip,port_Xplane,0,safe_local_y,jump);
case {'right','east'}
    setPosXPLANE(ip,port_Xplane,jump,safe_local_y,0);
case {'left','west'}
    setPosXPLANE(ip,port_Xplane,-jump,safe_local_y,0);
end
pause(long_pause);
end

function [Lat, Lon, NewX, NewZ, sc] = moveWithinSector(ip, port_Xplane, port_Matlab, dir, dist)
safe_local_y = 5000; % m
HOR = 0; VER = 1;
short_pause = 0.5;
    
[Lat, Lon, x, z] = getLatLonXZ(ip, port_Xplane, port_Matlab);
if any(~isfinite([dist x z])) || abs(dist)>50000
    disp('[setCoord] weird distance detected');
    return
end

% Move the plane and check if sector has changed
switch dir
    case {'x', 'X'}
        dir_code = HOR; % Horizontal
        setPosXPLANE(ip,port_Xplane,x+dist,safe_local_y,z);
        pause(short_pause);
        [Lat, Lon, NewX, NewZ] = getLatLonXZ(ip, port_Xplane, port_Matlab);
        sc = sectorHasChanged(NewX, NewZ, x+dist, z);
    case {'z', 'Z'}
        dir_code = VER; % Vertical
        setPosXPLANE(ip,port_Xplane,x,safe_local_y,z+dist);
        pause(short_pause);
        [Lat, Lon, NewX, NewZ] = getLatLonXZ(ip, port_Xplane, port_Matlab);
        sc = sectorHasChanged(NewX, NewZ, x, z+dist);
end

% If Sector changed, then correct it
if sc
    warning('Sector changed, staying at: (%.2f, %.2f) ',x,z)
    if dir_code == HOR
        if NewX < 0
            moveFromSector(ip, port_Xplane, 'west');
        else
            moveFromSector(ip, port_Xplane, 'east');
        end
    elseif dir_code == VER
        if NewZ < 0
            moveFromSector(ip, port_Xplane, 'north');
        else
            moveFromSector(ip, port_Xplane, 'south');
        end
    end
    setPosXPLANE(ip,port_Xplane,x,safe_local_y,z);
end
end

function ch = sectorHasChanged(new_x, new_z, x, z)
% Check if the sector has changed
if sqrt((x-new_x)^2+(z-new_z)^2) > 1000
    ch = true;
else
    ch = false;
end
    
end

function [Lat, Lon] = getLatLon(ip, port_Xplane, port_Matlab)
short_pause = 0.3;
trials  = 4;
intent  = 1;
success = false;
while intent <= trials && ~success
    try
        [Lat, Lon, ~, ~, ~, ~] = getPosXPLANE(ip, port_Xplane, port_Matlab);
        if all(isfinite([Lat, Lon]))
            success = true;
        else
            pause(short_pause);
        end
    catch err
        intent = intent + 1;
        pause(short_pause);
            if intent == 4
                fprintf(2,'Connection Lost\n');
                rethrow(err)
            end
    end
end
end

function [X, Z, Y] = getXZ(ip, port_Xplane, port_Matlab)
short_pause = 0.3;
trials = 4;
intent = 1;
success = false;
while intent <= trials && ~success
    try
        [~, ~, ~, X, Y, Z] = getPosXPLANE(ip, port_Xplane, port_Matlab);
        if all(isfinite([X, Z]))
            success = true;
        else
            pause(short_pause);
        end
    catch err
        intent = intent + 1;
        pause(short_pause);
            if intent == 4
                fprintf(2,'Connection Lost\n');
                rethrow(err)
            end
    end
end
end

function [Lat, Lon, X, Z] = getLatLonXZ(ip, port_Xplane, port_Matlab)
short_pause = 0.3;
trials = 4;
intent = 1;
success = false;
while intent <= trials && ~success
    try
        [Lat, Lon, ~, X, ~, Z] = getPosXPLANE(ip, port_Xplane, port_Matlab);
        if all(isfinite([Lat, Lon, X, Z]))
            success = true;
        else
            pause(short_pause);
        end
    catch err
        intent = intent + 1;
        pause(short_pause);
            if intent == 4
                fprintf(2,'Connection Lost\n');
                rethrow(err)
            end
    end
end
end
    