function setOrientationXPLANE(ip,port,v,roll,pitch,heading)
% Modify Xplane plane orientation via UDP :
% Input:
%   -> X-Plane IP         (string)
%   -> Port               (integer)
%   -> TAS                (knot)
%   -> Roll/Pitch/Heading (Degrees)
% * Possible improvement: read local_v (x,y,z) and angles phi, psi, theta
% * and rotate speed vectors accordingly

kt2m = 1852;

S = [-1 0 0; 0 1 0; 0 0 1]; % Correct X-Plane coord system
TAS = v *kt2m/3600;         % Must be in m/s

phi   = roll   * pi /180; % X
theta = pitch  * pi /180; % Y
psi   = heading* pi /180; % Z

vTAS = S*mrotY(psi)*mrotX(theta)*[0 0 -TAS]';% Coherent speed vector
q = angle2quat(phi,theta,psi,'XYZ');

variables = {'local_vx', 'local_vy', 'local_vz'};

ssock = java.net.DatagramSocket();
% Set speed
UDPsendDREFfloat(ssock,ip,port,'sim/flightmodel/position/local_vx',vTAS(1));
UDPsendDREFfloat(ssock,ip,port,'sim/flightmodel/position/local_vy',vTAS(2));
UDPsendDREFfloat(ssock,ip,port,'sim/flightmodel/position/local_vz',vTAS(3));

for i = 1:4
    dataref= ['sim/flightmodel/position/q[' num2str(i-1) ']'];
    value = q(i);
    UDPsendDREFfloat(ssock,ip,port,dataref,value);
end


ssock.close();

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function UDPsendDREFfloat(ssock,ip,port,dataref,value)
send_msg=uint8(zeros(1,509));
send_msg(1:4)=uint8('DREF');
send_msg(5)=uint8(0);

value=single(value);
value_bytes=typecast(value,'uint8');
len=6;
for j=1:4
    send_msg(len)=value_bytes(j);
    len=len+1;
end
dataref_bytes=uint8(dataref);
for j=1:length(dataref)
    send_msg(len)=dataref_bytes(j);
    len=len+1;
end

send_pkt = java.net.DatagramPacket(send_msg, size(send_msg,2),java.net.InetAddress.getByName(ip),port);
ssock.send(send_pkt);
clear send_pkt;
end

function m = mrotX(theta)
m = [1      0            0    ;...
     0  cos(theta) -sin(theta);...
     0  sin(theta)  cos(theta)];
end

function m = mrotY(psi)
m = [ cos(psi) 0 sin(psi);...
          0    1    0     ;...
     -sin(psi) 0 cos(psi)];
end