function setWeatherXPLANE(ip,port,data)
% Asensio Lorenzo 15/06/16

% Modify Xplane plane weather variables via UDP :
%
% Section: sim/weather/
% wind_altitude_msl_m[0]	int		meters >= 0	The center altitude of this layer of wind in MSL meters.
% wind_altitude_msl_m[1]	int		meters >= 0	The center altitude of this layer of wind in MSL meters.
% wind_altitude_msl_m[2]	int		meters >= 0	The center altitude of this layer of wind in MSL meters.
% wind_direction_degt[0]	float	[0 - 360)	The direction the wind is blowing from in degrees from true north c lockwise.
% wind_direction_degt[1]	float	[0 - 360)	The direction the wind is blowing from in degrees from true north c lockwise.
% wind_direction_degt[2]	float	[0 - 360)	The direction the wind is blowing from in degrees from true north c lockwise.
% wind_speed_kt[0]	    int		kts >= 0	The wind speed in knots.
% wind_speed_kt[1]	    int		kts >= 0	The wind speed in knots.
% wind_speed_kt[2]        int		kts >= 0	The wind speed in knots.
% 
% ip   : IP address of Xplane / String / Example: '127.0.0.1'
% port : port of Xplane / Integer / Example: 9005

section = 'sim/weather/';
allowed_dr = {...
    'wind_altitude_msl_m',...
    'wind_direction_degt',...
    'wind_speed_kt'};

% Open the receiving socket
ssock = java.net.DatagramSocket();
fn = fieldnames(data); % Read field names of the structure
if iscell(fn)          % Check if it has only one field
    l = length(fn);
else
    l = 1;
end

for i = 1:l                     
    if any(strcmp(fn{i},allowed_dr)) 
        c_values = data.(fn{i});        
        n_cells  = length(c_values);    
        for j = 1:n_cells               
            if ~isempty(c_values{j})        
                dataref= [section fn{i}...  
                    '[' num2str(j-1) ']'];  
                value = c_values{j};        
                UDPsendDREFfloat(ssock,ip,port,dataref,value);
            end
        end
    end
end

ssock.close();

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function UDPsendDREFfloat(ssock,ip,port,dataref,value)
send_msg=uint8(zeros(1,509));
send_msg(1:4)=uint8('DREF');
send_msg(5)=uint8(0);

value=single(value);
value_bytes=typecast(value,'uint8');
len=6;
for j=1:4
    send_msg(len)=value_bytes(j);
    len=len+1;
end
dataref_bytes=uint8(dataref);
for j=1:length(dataref)
    send_msg(len)=dataref_bytes(j);
    len=len+1;
end
send_pkt = java.net.DatagramPacket(send_msg, size(send_msg,2),java.net.InetAddress.getByName(ip),port);
ssock.send(send_pkt);
clear send_pkt;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function UDPsendDREFchar(ssock,ip,port,dataref,char)
send_msg=uint8(zeros(1,509));
send_msg(1:4)=uint8('DREF');
send_msg(5)=uint8(0);

value_bytes=typecast(char,'uint8');

send_msg(9)=value_bytes;
len=10;
dataref_bytes=uint8(dataref);
for j=1:length(dataref)
    send_msg(len)=dataref_bytes(j);
    len=len+1;
end
send_pkt = java.net.DatagramPacket(send_msg, size(send_msg,2),java.net.InetAddress.getByName(ip),port);
ssock.send(send_pkt);
clear send_pkt;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Other useful functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function send_msg=UDPsendDATA(ip,port,index,data)
ssock = java.net.DatagramSocket();  
n=length(index);
send_msg=uint8(zeros(1,41));
send_msg(1:4)=uint8('DATA');
send_msg(5)=uint8(0);

index=uint32(index);
data=single(data);

for j=1:n
    index_bytes=typecast(index(j),'uint8');
    len=6;
    for k=1:4
        send_msg(len)=index_bytes(k);
        len=len+1;
    end
    for h=1:8
        data_bytes=typecast(data((j-1)*8+h),'uint8');
        for k=1:4
            send_msg(len)=data_bytes(k);
            len=len+1;
        end
    end
    send_pkt = java.net.DatagramPacket(send_msg, 41, java.net.InetAddress.getByName(ip),port);
    ssock.send(send_pkt);
end
close(ssock);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function UDPsendCHAR(ip,port,char)
ssock = java.net.DatagramSocket();  
send_msg=uint8(zeros(1,6));
send_msg(1:4)=uint8('CHAR');
send_msg(5)=uint8(0);
send_msg(6)=uint8(char);
send_pkt = java.net.DatagramPacket(send_msg, size(send_msg,2), java.net.InetAddress.getByName(ip),port);
ssock.send(send_pkt);
close(ssock);
end